@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2017
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -17,56 +17,34 @@
 // It was later modified by Benjamin Schroeter and others.
 
 #include "polymake/polytope/minkowski_sum_fukuda.h"
+#include "polymake/Vector.h"
+#include "polymake/Matrix.h"
+#include "polymake/ListMatrix.h"
+#include "polymake/Graph.h"
+#include "polymake/hash_set"
+#include "polymake/list"
 
 namespace polymake { namespace polytope {
 
-namespace {
-   // the following template typedefs are deferred until c++11's "using"
-   
-   // template<typename E> typedef hash_set<Vector<E>> vertex_list;
-   // template<typename E> typedef std::vector<Vector<E>> clist; 
-   // template<typename E> typedef Array<Matrix<E>> matrix_list;
+template <typename E>
+using vertex_list = hash_set<Vector<E>>;
+template <typename E>
+using matrix_list = Array<Matrix<E>>;
+using graph_list = Array<Graph<Undirected>> ;
 
-   typedef Array<Graph<Undirected> > graph_list;
+template <typename E>
+Matrix<E> list2matrix(const vertex_list<E>& v)
+{
+   auto v_it = v.begin();
+   return Matrix<E>(v.size(), v_it->dim(), v_it);
 }
 
-/* converts a list to a matrix
- *
- */
-template<typename E>
-Matrix<E> list2matrix(const hash_set<Vector<E> >& v) {
-   const int dim=(v.begin())->size();
-   Matrix<E> A(v.size(),dim);
-
-   int i=0;
-   for (typename hash_set<Vector<E> >::const_iterator it=v.begin(); it!=v.end(); ++it, ++i) {
-      A.row(i)=*it;
-   }
-   return A;
-}
-
-/* converts a list to a matrix
- *
- */
-template<typename E>
-Matrix<E> list2matrix(const typename std::vector<Vector<E> >& v) {
-   const int dim=(v.begin())->size();
-   Matrix<E> A(v.size(),dim);
-
-   int i=0;
-   for (typename std::vector<Vector<E> >::const_iterator it=v.begin(); it!=v.end(); ++it, ++i) {
-      A.row(i)=*it;
-   }
-   return A;
-}
-
-
 /* LP solver uses cdd for Rational and Integer, and TOSimplex for other types
  *
  * returns optimal solution
  * called by Adj
  */
-template<typename E>
+template <typename E>
 Vector<E> solve_lp(const Matrix<E>& constraints, const Vector<E>& objective) {
    typedef typename choose_solver<E>::solver Solver;
    Solver solver;
@@ -82,7 +60,7 @@ Vector<E> solve_lp(const Matrix<E>& constraints, const Vector<E>& objective) {
  * called by initial, local_search and addition
  */
 template<typename E>
-Vector<E> components2vector(const Array<int>& components, const Array<Matrix<E> >& polytopes){
+Vector<E> components2vector(const Array<int>& components, const matrix_list<E>& polytopes){
    Vector<E> result(polytopes[0].row(0).size());
    int i=0;
    for(Array<int>::const_iterator it=components.begin(); it!=components.end(); ++it, ++i)       {
@@ -95,25 +73,24 @@ Vector<E> components2vector(const Array<int>& components, const Array<Matrix<E>
 /* computes a canonical vector in the relative interior of the normal cone N(v,P)
  * (can also return a vector [v,w], where w is a slightly perturbed variant of v)
  */
-template<typename E>
-Vector<E> canonical_vector(const int k, const Array<int>& components, const Array<Matrix<E> >& polytopes, const graph_list& graphs) {
-   typename std::vector<Vector<E> > c;  // constraints
-   for (int j=0; j<k; ++j) 
-      for (Entire<Graph<>::adjacent_node_list>::const_iterator it=entire(graphs[j].adjacent_nodes(components[j])); !it.at_end(); ++it) 
-         c.push_back(polytopes[j].row(*it)-polytopes[j].row(components[j]));
+template <typename E>
+Vector<E> canonical_vector(const int k, const Array<int>& components, const matrix_list<E>& polytopes, const graph_list& graphs) {
+   ListMatrix<Vector<E>> c;  // constraints
+   for (int j=0; j<k; ++j) {
+      for (auto it=entire(graphs[j].adjacent_nodes(components[j])); !it.at_end(); ++it)
+         c /= polytopes[j].row(*it)-polytopes[j].row(components[j]);
+   }
+   const E max_coef(accumulate(attach_operation(concat_rows(c), operations::abs_value()), operations::max()));
 
    // constraints of the form e_j(v_j,i)^T \lambda + \lambda_0 \leq 0
-   const Matrix<E> c1=list2matrix(c);
-   const int dim=c1.cols();
-   const Vector<E> c3=ones_vector<E>(c.size());
-   const Matrix<E> c2 = (c1 | -c3);
+   const int dim=c.cols();
+   c |= -ones_vector<E>(c.rows());
 
    // bounds for \lambda_i: -1\leq\lambda_i\leq 1
-   const Vector<E> b1(ones_vector<E>(dim));
-   const Matrix<E> b2(unit_matrix<E>(dim));
-   const Matrix<E> b= ((b1 | b2) / (b1 | -b2));
+   const Matrix<E> b= ((ones_vector<E>(dim) | unit_matrix<E>(dim)) /
+                       (ones_vector<E>(dim) | -unit_matrix<E>(dim)));
 
-   const Matrix<E> constraints(c2/b);
+   const Matrix<E> constraints(c/b);
    const Vector<E> obj(unit_vector<E>(dim+1,dim));      // objective function
    const Vector<E> r = solve_lp(constraints, obj);
 
@@ -124,8 +101,7 @@ Vector<E> canonical_vector(const int k, const Array<int>& components, const Arra
    Vector<E> eps(x.size());
    // as d is the minimal distance, we need to factor out the maximal
    // coefficient of the contraints times the dimension, to make sure
-   // the pertubation is small enough.
-   E max_coef(accumulate(attach_operation(concat_rows(c1), operations::abs_value()), operations::max()));
+   // the perturbation is small enough.
    eps[1] = d/(2*dim*max_coef);
    for (int i=2; i<eps.size(); ++i)
       eps[i] = eps[i-1]/E(-i);
@@ -170,7 +146,7 @@ bool parallel_edges(const Vector<E>& e1, const Vector<E>& e2) {
  * called by local_search and local_search_compare
  */
 template<typename E>
-Vector<E> search_direction(const int k, const Vector<E>& c_st1, const Vector<E>& c_st2, const Array<int>& components, const Array<Matrix<E> >& polytopes, const graph_list& graphs) {
+Vector<E> search_direction(const int k, const Vector<E>& c_st1, const Vector<E>& c_st2, const Array<int>& components, const matrix_list<E>& polytopes, const graph_list& graphs) {
    // computes canonical vector for v
    const Vector<E> c_eps = canonical_vector(k, components, polytopes, graphs);
    const int size(c_eps.size()/2);
@@ -215,7 +191,7 @@ Vector<E> search_direction(const int k, const Vector<E>& c_st1, const Vector<E>&
 
 
 template <class E>
-Vector<E> local_search(const int k, const Vector<E>& c_st, const Vector<E>& c_st2, Array<int>& components, const Array<Matrix<E> >& polytopes, const graph_list& graphs) 
+Vector<E> local_search(const int k, const Vector<E>& c_st, const Vector<E>& c_st2, Array<int>& components, const matrix_list<E>& polytopes, const graph_list& graphs) 
 {
    //search normal Vector for the first intersected hyperplane (part of Normalcone)
    const Vector<E> hyperplane=search_direction(k, c_st, c_st2, components, polytopes, graphs);
@@ -239,8 +215,8 @@ Vector<E> local_search(const int k, const Vector<E>& c_st, const Vector<E>& c_st
  * called by addition
  * TODO: kann verbessert werden in dem die richtung von next-v genutzt wird um ein erstes theta vorzugeben
  */
-template<typename E>
-bool local_search_compare(const int k, const Vector<E>& v_st, const Vector<E>& c_st, const Vector<E>& c_st2, const Vector<E>& v, const Vector<E>& next, const Array<int>& next_components, const Array<Matrix<E> >& polytopes, const graph_list& graphs) 
+template <typename E>
+bool local_search_compare(const int k, const Vector<E>& v_st, const Vector<E>& c_st, const Vector<E>& c_st2, const Vector<E>& v, const Vector<E>& next, const Array<int>& next_components, const matrix_list<E>& polytopes, const graph_list& graphs) 
 {
    if (next==v_st) return 0;
 
@@ -253,32 +229,32 @@ bool local_search_compare(const int k, const Vector<E>& v_st, const Vector<E>& c
 /* searchs a adjacent edge of v in G(P) in direction e_s(v_s,r)
  *
  * returns 0 if there is no edge or the index (s,r) is not the smallest for this direction
- * subroutins: parallel_edges, list2matrix and solve_lp
+ * subroutines: parallel_edges and solve_lp
  * called by addition
  */
-template<typename E>
-bool Adj(const int k, Array<int>& next_components, const Array<int>& components, const int s, const int r, const graph_list& graphs, Array<Matrix<E> >& polytopes)
+template <typename E>
+bool Adj(const int k, Array<int>& next_components, const Array<int>& components, const int s, const int r, const graph_list& graphs, matrix_list<E>& polytopes)
 {
    const Vector<E> e_s = polytopes[s].row(r) - polytopes[s].row(components[s]); // =e_s(v_s,r)
-   Vector<E> e_j;                  // = e_j(v_j,i)
-   std::vector<Vector<E> > ineq_list;              // list of ineq. (3.6) e_j(v_j,i)^T lambda >=0
-   ineq_list.push_back(-e_s);      // and e_s(v_s,r)^T lambda < 0
+   Vector<E> e_j;                      // = e_j(v_j,i)
+   ListMatrix<Vector<E>> ineq_list;    // list of ineq. (3.6) e_j(v_j,i)^T lambda >=0
+   ineq_list /= -e_s;                  // and e_s(v_s,r)^T lambda < 0
    next_components=components;
-   for(int j=0;j<k;++j) {
-      for (Entire<Graph<>::adjacent_node_list>::const_iterator it = entire(graphs[j].adjacent_nodes(components[j])); !it.at_end(); ++it) {
+   for (int j=0; j<k ;++j) {
+      for (auto it = entire(graphs[j].adjacent_nodes(components[j])); !it.at_end(); ++it) {
          e_j = polytopes[j].row(*it) - polytopes[j].row(components[j]);
          if (parallel_edges(e_s, e_j)) {
             // returns 0 if there is an index j smaller than s in same direction
             if (j<s) return 0;
             next_components[j]=*it;
          } else {
-            ineq_list.push_back(e_j);
+            ineq_list /= e_j;
          }
       }
    }
    // add another variable mu in the first inequality
    // (the ineq will be strict iff mu is positive)
-   const Matrix<E> m = ( (list2matrix(ineq_list) ) | -unit_vector<E>(ineq_list.size(), 0) );
+   const Matrix<E> m( ineq_list | -unit_vector<E>(ineq_list.rows(), 0) );
    const int l=m.cols()-1;
    // one more ineq to bound lambda from above
    Vector<E> b = unit_vector<E>(m.cols(),0);
@@ -296,14 +272,15 @@ bool Adj(const int k, Array<int>& next_components, const Array<int>& components,
  * subroutins: Adj, local_search, local_search_compare and components2vector
  * called by minkowski_sum_fukuda
  */
-template<typename E>
-hash_set<Vector<E> > addition(const int k, const Vector<E>& v_st, const Vector<E>& c_st, const Vector<E>& c_st2, Array<int>& components, const graph_list& graphs, Array<Matrix<E> >& polytopes){
-   hash_set<Vector<E> > vertices; //return value
+template <typename E>
+vertex_list<E> addition(const int k, const Vector<E>& v_st, const Vector<E>& c_st, const Vector<E>& c_st2, Array<int>& components, const graph_list& graphs, matrix_list<E>& polytopes)
+{
+   vertex_list<E> vertices; //return value
    Vector<E> next;
    Array<int> next_components(k), u_components(k);
    Vector<E> v = v_st;
    int j=0;                     // iterates through all input-polytopes
-   Entire<Graph<>::adjacent_node_list>::const_iterator it=entire(graphs[j].adjacent_nodes(components[j]));      //iterates through all neighbors in P_j
+   auto it=entire(graphs[j].adjacent_nodes(components[j]));      //iterates through all neighbors in P_j
    vertices.insert(v_st);
 
    while (j!=k or v!=v_st) {
@@ -437,7 +414,7 @@ int lex_max(const int a, const int b, const Matrix<E>& mat){
  */
 template <typename E>
 void initialize(const Array<perl::Object>& summands, const int k, graph_list& graphs,
-                Array<Matrix<E>>& polytopes, Array<int>& components, Vector<E>& v_st, Vector<E>& c_st, Vector<E>& c_st2)
+                matrix_list<E>& polytopes, Array<int>& components, Vector<E>& v_st, Vector<E>& c_st, Vector<E>& c_st2)
 {
    //initialize graphs and polytopes:
    int j=0;
@@ -497,10 +474,10 @@ Matrix<E> minkowski_sum_vertices_fukuda(const Array<perl::Object>& summands)
    Vector<E> c_st2;     // alternative for c* (if c is parallel to c_st)
    Array<int> components(k);    // j-th entry will contain the number of a vertex in P_j  s.t. v = v(1)+v(2)+...+v(k) where v(j)= P_j(comp(j))
    graph_list graphs(k);                // stores all Graphs from the input Polytopes P_j
-   Array<Matrix<E>> polytopes(k);      // stores matrices s.t. the i-th entry is a discribtion of P_j by vertices
+   matrix_list<E> polytopes(k);      // stores matrices s.t. the i-th entry is a discribtion of P_j by vertices
 
    initialize(summands, k, graphs, polytopes, components, v_st, c_st, c_st2);
-   hash_set<Vector<E>> vertices = addition(k, v_st, c_st, c_st2, components, graphs, polytopes);
+   vertex_list<E> vertices = addition(k, v_st, c_st, c_st2, components, graphs, polytopes);
    // Output = listed coordinates of the vertices of P, (the first vertex will be v*)
    return list2matrix(vertices);
 }
@@ -573,7 +550,8 @@ UserFunctionTemplate4perl("# @category Producing a polytope from scratch"
                           "# @param Matrix<Scalar> M"
                           "# @option Bool centered_zonotope default 1"
                           "# @return Matrix<E>"
-                          "# @example The following stores the vertices of a parallelogram with the origin as its vertex barycenter and prints them:"
+                          "# @example [nocompare]"
+                          "# The following stores the vertices of a parallelogram with the origin as its vertex barycenter and prints them:"
                           "# > $M = new Matrix([[1,1,0],[1,1,1]]);"
                           "# > print zonotope_vertices_fukuda($M);"
                           "# | 1 0 -1/2"
