@@ -1,7 +1,6 @@
-#  Copyright (c) 1997-2019
-#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
-#  Technische Universität Berlin, Germany
-#  https://polymake.org
+#  Copyright (c) 1997-2018
+#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
+#  http://www.polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -72,17 +71,17 @@ my %rules_to_wake;
 
 # private
 sub store_rule_to_wake {
-   my ($self, $filename, $ext, $rule_key, $on_rule) = splice @_, 0, 5;
+   my ($self, $filename, $ext, $rule_key, $on_rule)=splice @_, 0, 5;
    for (;;) {
       my $list;
       if ($on_rule) {
          my ($prereq_app, $prereq_rule_key) = splice @_, 0, 2  or  last;
-         $list = ($rules_to_wake{$prereq_app}->{$prereq_rule_key} //= [ ]);
+         $list=($rules_to_wake{$prereq_app}->{$prereq_rule_key} //= [ ]);
       } else {
          my $prereq_ext = shift @_  or  last;
          $list=($rules_to_wake{$prereq_ext} //= [ ]);
       }
-      if (@$list && $list->[-2] == $self) {
+      if (@$list && $list->[-2]==$self) {
          push @{$list->[-1]}, $filename, $ext, $rule_key;
       } else {
          push @$list, $self, [ $filename, $ext, $rule_key ];
@@ -93,30 +92,29 @@ sub store_rule_to_wake {
 # private
 sub do_reconfigure {
    my ($self, $list, $filename, $ext, $rule_key);
-   my @woken = @_;
-   while (($self, $list) = splice @woken, 0, 2) {
-      my $new_to_wake = @woken;
+   my @woken=@_;
+   while (($self, $list)=splice @woken, 0, 2) {
+      my $new_to_wake=@woken;
 
-      local if (!defined($self->compile_scope)) {
-         local unshift @INC, $self;
-         local scalar $self->compile_scope = new Scope();
-         local ref $self->custom->tied_vars = [];
-         $self->compile_scope->cleanup->{$self->custom} = undef;
-      }
-      while (($filename, $ext, $rule_key) = splice @$list, 0, 3) {
+      defined($self->compile_scope) or
+        local_unshift(\@INC, $self),
+        local_scalar($self->compile_scope, new Scope()),
+        local_array($self->custom->tied_vars, [ ]),
+        $self->compile_scope->cleanup->{$self->custom}=undef;
+
+      while (($filename, $ext, $rule_key)=splice @$list, 0, 3) {
          my $rc;
-         local $extension = $ext if $ext != $extension;
-         if (defined (my $code = $self->rule_code->{$rule_key})) {
-            my $reconf = new Reconf($self, false);
-            no strict 'refs';
-            local ref *{$self->pkg."::self"} = sub { $reconf };
-            if (is_object (my $credit = $self->credits_by_rulefile->{$rule_key})) {
-               $credit->shown &= ~Rule::Credit::hide;
+         local $extension=$ext if $ext != $extension;
+         if (defined (my $code=$self->rule_code->{$rule_key})) {
+            my $reconf=new Reconf($self);
+            local_sub(get_symtab($self->pkg)->{self}, sub { $reconf });
+            if (is_object (my $credit=$self->credits_by_rulefile->{$rule_key})) {
+               $credit->shown &= ~$Rule::Credit::hide;
             }
             &$code;
-            $rc = $self->rulefiles->{$rule_key};
+            $rc=$self->rulefiles->{$rule_key};
          } else {
-            $rc = parse_rulefile($self, $rule_key, $filename);
+            $rc=parse_rulefile($self, $rule_key, $filename);
          }
          if ($rc && defined (my $to_wake=$rules_to_wake{$self})) {
             if (defined ($to_wake=delete $to_wake->{$rule_key})) {
@@ -126,12 +124,12 @@ sub do_reconfigure {
       }
 
       $_->finalize for @{$self->rules_to_finalize};
-      @{$self->rules_to_finalize} = ();
+      @{$self->rules_to_finalize}=();
 
-      for (; $new_to_wake < $#woken; $new_to_wake += 2) {
-         ($self, $list) = @woken[$new_to_wake, $new_to_wake + 1];
-         for (my $i = 0; $i < @$list; $i += 3) {
-            ($filename, $ext, $rule_key) = @$list[$i..$i+2];
+      for (; $new_to_wake<$#woken; $new_to_wake+=2) {
+         ($self, $list)=@woken[$new_to_wake, $new_to_wake+1];
+         for (my $i=0; $i<@$list; $i+=3) {
+            ($filename, $ext, $rule_key)=@$list[$i..$i+2];
             delete $self->configured->{$rule_key};
          }
       }
@@ -139,10 +137,10 @@ sub do_reconfigure {
 }
 ###############################################################################################
 sub lookup_rulefile_or_key {
-   my ($self, $rulefile) = @_;
+   my ($self, $rulefile)=@_;
    if ($rulefile =~ s/\@(.*)$//) {
       # specified by the key including extension URI
-      local $extension = $Extension::registered_by_URI{$1};
+      local $extension=$Extension::registered_by_URI{$1};
       if (defined $extension) {
          lookup_rulefile($self, $rulefile, 1);
       } else {
@@ -154,17 +152,17 @@ sub lookup_rulefile_or_key {
 }
 ###############################################################################################
 sub reconfigure {
-   my ($self, $rulefile) = @_;
+   my ($self, $rulefile)=@_;
    if ($rulefile =~ m{^($id_re)::(.*)}o) {
       reconfigure($self->used->{$1} || die("application $1 is not declared as USE'd or IMPORT'ed in the current application\n"), "$2");
 
    } elsif (my ($filename, $ext, $rule_key, $old_status)=&lookup_rulefile_or_key) {
-      if (defined (my $conf_status = $self->configured->{$rule_key})) {
-         if (is_string($conf_status) and my ($rules, $what) = $conf_status =~ /^0\#(?:(rule)|ext):(.*)/) {
-            my @prereq = split /\|/, $what;
+      if (defined (my $conf_status=$self->configured->{$rule_key})) {
+         if (is_string($conf_status) and my ($rules, $what)= $conf_status =~ /^0\#(?:(rule)|ext):(.*)/) {
+            my @prereq=split /\|/, $what;
             die( "rulefile $rulefile requires ",
-                  $rules ? ( @prereq > 1 ? "one of the rulefiles ".join(", ", @prereq) : "another rulefile $what" )
-                         : ( @prereq > 1 ? "one of extensions ".join(", ", @prereq) : "the extension $what" ),
+                  $rules ? ( @prereq>1 ? "one of the rulefiles ".join(", ", @prereq) : "another rulefile $what" )
+                         : ( @prereq>1 ? "one of extensions ".join(", ", @prereq) : "the extension $what" ),
                   " which must be reconfigured first;\n",
                   "in the case of success the desired rulefile might be enabled automatically.\n" );
          }
@@ -185,22 +183,20 @@ sub reconfigure {
 }
 ###############################################################################################
 sub unconfigure {
-   my ($self, $rulefile) = @_;
+   my ($self, $rulefile)=@_;
    if ($rulefile =~ m{^($id_re)::(.*)}o) {
       unconfigure($self->used->{$1} || die("application $1 is not declared as USE'd or IMPORT'ed in the current application"), "$2");
 
-   } elsif (my ($filename, $ext, $rule_key, $old_status) = &lookup_rulefile_or_key) {
-      my $config_state = $self->configured->{$rule_key};
-      if ($config_state > 0) {
-         if (defined (my $code = $self->rule_code->{$rule_key})) {
-            my $unconf = new Unconf($self);
-            no strict 'refs';
-            local ref *{$self->pkg."::self"} = sub { $unconf };
-            local $extension = $ext;
+   } elsif (my ($filename, $ext, $rule_key, $old_status)=&lookup_rulefile_or_key) {
+      my $config_state=$self->configured->{$rule_key};
+      if ($config_state>0) {
+         if (defined (my $code=$self->rule_code->{$rule_key})) {
+            my $unconf=new Unconf($self);
+            local_sub(get_symtab($self->pkg)->{self}, sub { $unconf });
             &$code;
          } else {
-            $self->configured->{$rule_key} = -$load_time;
-            $self->custom->set_changed;
+            $self->configured->{$rule_key}=-$load_time;
+            $self->custom->handler->need_save=1;
          }
       } elsif (defined $config_state) {
          die "rulefile $rulefile is already unconfigured\n";
@@ -219,20 +215,20 @@ sub valid_configured_entry {
       if ($rule_key =~ m{^/}) {
          -f $rule_key or do {
             delete $self->configured->{$rule_key};
-            $self->custom->set_changed;
-            false
+            $self->custom->handler->need_save=1;
+            0
          }
       } else {
-         my ($rulefile, $ext) = split /\@/, $rule_key;
+         my ($rulefile, $ext)=split /\@/, $rule_key;
          if ($ext
-             ? ($ext = $Extension::registered_by_URI{$ext},
+             ? ($ext=$Extension::registered_by_URI{$ext},
                 -f $ext->app_dir($self) . "/rules/$rulefile")
              : -f $self->top . "/rules/$rulefile") {
             !$ext || $ext->is_active;
          } else {
             delete $self->configured->{$rule_key};
-            $self->custom->set_changed;
-            false
+            $self->custom->handler->need_save=1;
+            0
          }
       }
    }
@@ -243,63 +239,62 @@ sub list_configured {
    grep {
       valid_configured_entry($self, $_) and
       ($state < 0
-       ? $self->configured->{$_} < 0 || $self->configured->{$_} eq "0" || $self->configured->{$_} =~ /\#$/ :
+       ? $self->configured->{$_}<0 || $self->configured->{$_} eq "0" || $self->configured->{$_} =~ /\#$/ :
        $state > 0
-       ? $self->configured->{$_} > 0
+       ? $self->configured->{$_}>0
        : $self->configured->{$_} !~ /^0\#/)
    } keys %{$self->configured}
 }
 ###############################################################################################
 sub first_credit_in {
-   my ($self, $rule_key) = @_;
+   my ($self, $rule_key)=@_;
    LoadPreamble::include_rule($self, $rule_key);
-   if (defined (my $credit = $self->credits_by_rulefile->{$rule_key})) {
+   if (defined (my $credit=$self->credits_by_rulefile->{$rule_key})) {
       if (is_object($credit)) {
          $credit
       } else {
-         $self->credits_by_rulefile->{$rule_key} = lookup_credit($self, $credit)
+         $self->credits_by_rulefile->{$rule_key}=lookup_credit($self, $credit)
       }
-   } elsif ($rule_key =~ /\@(bundled:\w+)$/) {
-      $Extension::registered_by_URI{$1}->credit
    } else {
       undef
    }
 }
 ###############################################################################################
 sub extend_in {
-   my ($self, $ext) = @_;
-   if ($self->installTop eq $ext->dir || list_index($self->extensions, $ext) >=0 ) {
+   my ($self, $ext)=@_;
+   if ($self->installTop eq $ext->dir || list_index($self->extensions, $ext)>=0) {
       die "Extension ", $ext->URI, " already contributes to application ", $self->name, "\n";
    }
    if ($self->installTop ne $InstallTop) {
-      my $foundation_ext = $Extension::registered_by_dir{$self->installTop};
-      if (list_index($foundation_ext->requires, $ext) >= 0) {
+      my $foundation_ext=$Extension::registered_by_dir{$self->installTop};
+      if (list_index($foundation_ext->requires, $ext)>=0) {
          die "Application ", $self->name, " is founded in extension ", $foundation_ext->URI, " installed at ", $self->installTop, "\n",
              "which has the specified extension ", $ext->URI, " among its direct or indirect prerequisites.\n",
              "Adding here a contribution to ", $app->name, " would introduce a forbidden cyclic dependency between both extensions.\n",
              "You might consider moving the foundation of ", $app->name, " into extension ", $ext->URI, "\n",
              "or into a third extension, independent on these ones.\n";
       }
-      if (list_index($ext->requires, $foundation_ext) < 0) {
+      if (list_index($ext->requires, $foundation_ext)<0) {
          $ext->add_prerequisites($foundation_ext);
       }
    }
 
-   my $source_tree = $ext->get_source_tree;
+   my $vcs=$ext->get_source_VCS;
 
-   my $app_dir = $ext->app_dir($self);
-   $source_tree->make_dir(map { "$app_dir/$_" } qw(src include perllib rules scripts testsuite));
+   my $app_dir=$ext->app_dir($self);
+   $vcs->make_dir(map { "$app_dir/$_" } qw(src include perllib rules scripts testsuite));
    if ($ext->is_bundled) {
       open my $noexport, ">", "$app_dir/.noexport";
       print $noexport "testsuite test\n";
       close $noexport;
+      $vcs->add_file("$app_dir/.noexport");
    }
 
    # create include symlink
-   my $link_dir = $ext->dir."/include/apps/polymake";
-   -d $link_dir or $source_tree->make_dir($link_dir);
-   my $app_name = $self->name;
-   my $link = "$link_dir/$app_name";
+   my $link_dir=$ext->dir."/include/apps/polymake";
+   -d $link_dir or $vcs->make_dir($link_dir);
+   my $app_name=$self->name;
+   my $link="$link_dir/$app_name";
    if (-l $link) {
       unlink $link
         or die "Can't remove obsolete link $link: $!\n";
@@ -308,18 +303,19 @@ sub extend_in {
    }
    symlink "../../../apps/$app_name/include", $link
       or die "Can't create a link $link: $!\n";
+   $vcs->add_file($link);
 }
 ###############################################################################################
 sub found {
-   require Polymake::SourceTree;
-   (undef, my ($name, $ext)) = @_;
-   my $src_tree = $ext ? $ext->get_source_tree : new SourceTree($InstallTop);
-   my $app_dir = ($ext ? $ext->dir : $InstallTop)."/apps/$name";
-   $src_tree->make_dir(map { "$app_dir/$_" } qw(src include perllib rules scripts testsuite));
+   (undef, my ($name, $ext))=@_;
+   my $vcs= $ext ? $ext->get_source_VCS : $CoreVCS;
+   my $app_dir=($ext ? $ext->dir : $InstallTop)."/apps/$name";
+   $vcs->make_dir(map { "$app_dir/$_" } qw(src include perllib rules scripts testsuite));
    unless ($ext) {
       open my $noexport, ">", "$app_dir/.noexport";
       print $noexport "testsuite test\n";
       close $noexport;
+      $vcs->add_file("$app_dir/.noexport");
    }
 }
 ###############################################################################################
@@ -331,62 +327,53 @@ use Polymake::Struct (
 
 # pretend the last configuration step in the preamble to fail
 sub configure {
-   my ($self, $code, $rule_key, $line) = @_;
+   my ($self, $code, $rule_key, $line)=@_;
    $line < $self->preamble_end->{$rule_key};
 }
 
-*include_required = \&configure;
-*require_ext = \&configure;
+*include_required=\&configure;
+*require_ext=\&configure;
 
 sub include_rule_block {}
 
 sub add_credit {
-   my $self = shift;
-   my $credit = $self->SUPER::add_credit(@_, 1);
-   $credit->shown = Rule::Credit::hide;
+   my $self=shift;
+   my $credit=$self->SUPER::add_credit(@_, 1);
+   $credit->shown=$Rule::Credit::hide;
    $credit
 }
 
 sub include_rule {
-   my ($self, $rule_key) = @_;
-   if (my $code = $self->rule_code->{$rule_key}) {
-      my $credit = $self->credits_by_rulefile->{$rule_key};
+   my ($self, $rule_key)=@_;
+   if (my $code=$self->rule_code->{$rule_key}) {
+      my $credit=$self->credits_by_rulefile->{$rule_key};
       if (defined($credit) && !is_object($credit)) {
-         my $reread = new Reconf($self, true);
-         no strict 'refs';
-         local ref *{$self->pkg."::self"} = sub { $reread };
+         local_bless($self, __PACKAGE__);
+         my $reread=new Reconf($self, 1);
+         local_unshift(\@INC, $self);
+         local_sub(get_symtab($self->pkg)->{self}, sub { $reread });
          &$code;
       }
    } else {
-      local bless $self;
-      my ($rulefile, $ext_URI) = split /\@/, $rule_key;
-      local $extension = $Extension::registered_by_URI{$ext_URI} if $ext_URI;
-      my $conf = $self->configured->{$rule_key};
-      if ($conf > 0 || !defined($conf)) {
-         local $Shell = new NoShell();
+      my ($rulefile, $ext_URI)=split /\@/, $rule_key;
+      local $extension=$Extension::registered_by_URI{$ext_URI} if $ext_URI;
+      my $conf=$self->configured->{$rule_key};
+      if ($conf>0 || !defined($conf)) {
+         local $Shell=new NoShell();
          include_rules($self, $rulefile);
       } else {
-         my $save_buf;
          delete local $self->rulefiles->{$rule_key};
-         tied(%{$self->configured})->hide($rule_key, $save_buf);
+         delete local $self->configured->{$rule_key};
+         local_bless($self, __PACKAGE__);
          include_rules($self, $rulefile);
       }
    }
 }
-###############################################################################################
-package Polymake::Core::Application::CppDummy;
-
-sub new {
-   state $dummy=bless [ ], $_[0];
-}
-
-sub start_loading {}
-
 ###############################################################################################
 package Polymake::Core::Application::Reconf;
 
 use Polymake::Struct (
-   [ new => '$$' ],
+   [ new => '$;$' ],
    [ '$application' => '#1' ],
    [ '$simulate_failure' => '#2' ],
 );
@@ -394,34 +381,32 @@ use Polymake::Struct (
 sub start_preamble { shift }
 sub preamble_end { $_[0]->application->preamble_end }
 
-sub cpp { new CppDummy; }
-
 sub add_credit {
-   my $self = shift;
-   my $credit = $self->application->add_credit(@_, 1);
+   my $self=shift;
+   my $credit=$self->application->add_credit(@_, 1);
    if ($self->simulate_failure && $_[1] =~ /\S/) {
-      $credit->shown = Rule::Credit::hide;
+      $credit->shown=$Rule::Credit::hide;
    }
    $credit
 }
 
 sub add_custom {
-   my $self = shift;
-   if (!$self->simulate_failure  and  defined(my $var = $self->application->custom->re_tie(@_))) {
-      $var->extension = $extension;
+   my $self=shift;
+   if (!$self->simulate_failure  and  my $var=$self->application->custom->re_tie(@_)) {
+      $var->extension=$extension;
    }
 }
 
 sub close_preamble {
-   my ($app, $rule_key, $line, $success) = @_;
+   my ($app, $rule_key, $line, $success)=@_;
    if ($line >= $app->preamble_end->{$rule_key}) {
-      ($app->rulefiles->{$rule_key} &&= return 0) = $success;   # avoid repeated execution of the rulefile body
+      ($app->rulefiles->{$rule_key} &&= return 0)=$success;   # avoid repeated execution of the rulefile body
    }
    $success;
 }
 
 sub configure {
-   my ($self, $code, $rule_key, $line, $optional) = @_;
+   my ($self, $code, $rule_key, $line, $optional)=@_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -430,7 +415,7 @@ sub configure {
 }
 
 sub include_required {
-   my ($self, $block, $rule_key, $line) = @_;
+   my ($self, $block, $rule_key, $line)=@_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -439,7 +424,7 @@ sub include_required {
 }
 
 sub require_ext {
-   my ($self, $block, $rule_key, $line) = @_;
+   my ($self, $block, $rule_key, $line)=@_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -448,8 +433,8 @@ sub require_ext {
 }
 
 sub AUTOLOAD {
-   my $self = shift;
-   my ($method) = $AUTOLOAD =~ /::(\w+)$/;
+   my $self=shift;
+   my ($method)= $AUTOLOAD =~ /::(\w+)$/;
    $self->application->$method(@_);
 }
 
@@ -466,36 +451,32 @@ use Polymake::Struct (
 sub start_preamble { shift }
 sub preamble_end { $_[0]->application->preamble_end }
 
-sub cpp { new CppDummy; }
-
 sub add_credit {
-   my ($self, $product, $ext_URI, $text) = @_;
-   my $credit = $self->application->lookup_credit($product);
+   my ($self, $product, $ext_URI, $text)=@_;
+   my $credit=$self->application->lookup_credit($product);
    if ($text =~ /\S/) {
-      $credit->shown = Rule::Credit::hide;
+      $credit->shown=$Rule::Credit::hide;
    }
    $credit;
 }
 
 sub add_custom {
-   my ($self, $varname, $help_text, $state, $pkg) = @_;
+   my ($self, $varname, $help_text, $state, $pkg)=@_;
    $self->application->custom->unset($varname, $pkg) if $state;
 }
 
 # pretend the last configuration step in the preamble to fail
 sub configure {
-   my ($self, $code, $rule_key, $line) = @_;
+   my ($self, $code, $rule_key, $line)=@_;
    $line < $self->application->preamble_end->{$rule_key} or do {
-      $self->application->configured->{$rule_key} = -$load_time;
-      $self->application->custom->set_changed;
+      $self->application->configured->{$rule_key}=-$load_time;
+      $self->application->custom->handler->need_save=1;
       0
    }
 }
 
-*include_required = \&configure;
-*require_ext = \&configure;
-
-sub include_rule_block {}
+*include_required=\&configure;
+*require_ext=\&configure;
 
 ###############################################################################################
 package Polymake::Core::Extension;
@@ -561,10 +542,10 @@ sub obliterate {
             $app->custom->obliterate_extension($self);
             $app->cpp->obliterate_extension($self);
             my $had_configured;
-            while (my ($rule_key) = each %{$app->configured}) {
+            while (my ($rule_key)=each %{$app->configured}) {
                if (index($rule_key, '@'.$self->URI)==0) {
                   delete $app->configured->{$rule_key};
-                  $had_configured = true;
+                  $had_configured=1;
                }
             }
             if ($had_configured) {
@@ -598,14 +579,12 @@ sub may_obliterate {
 }
 ###############################################################################################
 sub new_bundled {
-   require Polymake::SourceTree;
    my ($pkg, $name)=@_;
    my $ext_dir="$InstallTop/bundled/$name";
    if (-d $ext_dir) {
       die "bundled extension $ext_dir already exists\n";
    }
-   my $src_tree=new SourceTree($InstallTop);
-   $src_tree->make_dir("$ext_dir/apps", "$ext_dir/include", "$ext_dir/support");
+   $CoreVCS->make_dir("$ext_dir/apps", "$ext_dir/include", "$ext_dir/support");
    open my $META, ">", "$ext_dir/polymake.ext"
      or die "can't create description file $ext_dir/polymake.ext: $!\n";
    print $META <<'.';
@@ -627,10 +606,12 @@ sub new_bundled {
 # URIs of other bundled extensions which are incompatible with this one.
 .
    close $META;
-   $src_tree->copy_file("$InstallTop/support/configure.pl.template", "$ext_dir/support/configure.pl.template");
+   $CoreVCS->add_file("$ext_dir/polymake.ext");
+   $CoreVCS->copy_file("$InstallTop/support/configure.pl.template", "$ext_dir/support/configure.pl.template");
    my $ext=new($pkg, $ext_dir, "bundled:$name", 1);
    $registered_by_URI{$ext->URI}=$ext;
    $registered_by_dir{$ext_dir}=$ext;
+   $ext->VCS=$CoreVCS;
    # at the beginning of its life, the extension does not depend on anything else nor requires any configuration
    unshift @BundledExts, $name;
    require Polymake::Configure;
@@ -673,9 +654,6 @@ otherwise choose a location outside $InstallTop
    foreach my $file (qw(configure.pl install.pl generate_ninja_targets.pl)) {
       File::Copy::copy("$InstallTop/support/$file.template", "$ext_dir/support");
    }
-   unless (-f "$ext_dir/.gitignore") {
-      File::Copy::copy("$InstallTop/support/gitignore.template", "$ext_dir/.gitignore");
-   }
    # create the skeleton of the build directory
    require Polymake::Configure;
    $ext->set_build_dir;
@@ -827,7 +805,6 @@ sub reconfigure {
    }
 
    require Polymake::Configure;
-   $self->set_build_dir;
    my $err=Configure::configure_extension($self, "--force", @config_options);
    if ($err ne "silent\n") {
       if ($err) {
@@ -883,14 +860,14 @@ All C++ components in the extension will be rebuilt at the start of the next pol
 # private:
 sub write_initial_description {
    my ($self)=@_;
-   my $src_tree=$self->get_source_tree;
+   my $vcs=$self->get_source_VCS;
 
    open my $descr, ">", $self->dir."/polymake.ext"
       or die "can't create extension description ", $self->dir, "/polymake.ext: $!\n";
    print $descr <<'.';
 # This is a description file of a polymake extension rooted in this directory.
 # Please refer to the documentation under
-#   https://polymake.org/doku.php/user_guide/extend/extensions
+#   http://www.polymake.org/doku.php/reference/extensions
 # for detailed explanations of its contents.
 
 # URI is a unique identifier assigned to this extension.
@@ -940,13 +917,18 @@ sub write_initial_description {
 .
    close $descr;
 
+   $vcs->add_file("polymake.ext");
+   $vcs->set_ignored("build", "build.*");
+
    if (-f (my $URIfile=$self->dir."/URI")) {
-      unlink $URIfile or die "can't remove $URIfile: $!\n";
+      $vcs->delete_file("URI");
    }
 
    unless (glob($self->dir."/support/configure.pl*")) {
+      require File::Copy;
       foreach my $filename (qw(configure generate_ninja_targets install)) {
-         $src_tree->copy_file("$InstallTop/support/$filename.pl.template", "support/$filename.pl.template");
+         File::Copy::copy("$InstallTop/support/$filename.pl.template", $self->dir."/support/$filename.pl.template");
+         $vcs->add_file("support/$filename.pl.template");
       }
    }
 }
@@ -973,7 +955,7 @@ sub add_prerequisites {
    }
 
    require Polymake::Configure;
-   Configure::rewrite_config_files($self->dir, RequireExtensions => $self->list_prerequisite_extensions);
+   Configure::rewrite_config_files($self->dir, "RequireExtensions", Configure::list_prerequisite_extensions($self));
 }
 ###############################################################################################
 
@@ -984,26 +966,17 @@ package Polymake::User;
 
 ###############################################################################################
 sub help {
-   my ($subject) = @_;
-   if (length($subject) == 0) {
-      print $Core::Help::core->display_text;
-      return;
+   my ($subject)=@_;
+   my $help_delim=$help_delimit ? "-------------------\n" : '';
+   my $app=$application;
+   if ($subject =~ /^($id_re)::/o && $app->used->{$1}) {
+      $subject=$';
+      $app=$app->used->{$1};
    }
-   my $help_delim = $help_delimit ? "-------------------\n" : '';
-   my $top_help;
-   if ($subject =~ m{^($id_re)(?:$|::|/)}o && defined(my $app = lookup Core::Application($1))) {
-      $subject = $';
-      $top_help = $app->help;
-   } elsif ($subject =~ m{^core/}) {
-      $subject = $';
-      $top_help = $Core::Help::core;
-   } else {
-      $top_help = $application->help;
-   }
-   if (my @topics = uniq( $top_help->get_topics($subject) )) {
+   if (my @topics=uniq( $app->help->get_topics($subject || "top") )) {
       my (@subcats, @subtopics, $need_delim);
       foreach my $topic (@topics) {
-         my $text = $topic->display_text;
+         my $text=$topic->display_text;
          if (length($text)) {
             print $help_delim if $need_delim++;
             print $text, "\n";
@@ -1011,7 +984,7 @@ sub help {
          foreach (@{$topic->toc}) {
             if ($topic->topics->{$_}->category & 1) {
                push @subcats, $_;
-            } elsif ($_ ne "any") {
+            } else {
                push @subtopics, $_;
             }
          }
@@ -1019,23 +992,24 @@ sub help {
       if (@subcats || @subtopics) {
          print $help_delim if $need_delim;
       }
-      my $fp = $topics[0]->full_path =~ s{^/}{}r || $topics[0]->name;
+      my $fp=substr($topics[0]->full_path,1);
+      $fp &&= " of $fp";
       if (@subcats) {
-         print "Categories of $fp:\n", (join ", ", sort @subcats), "\n";
+         print "Categories$fp:\n", (join ", ", sort @subcats), "\n";
       }
       if (@subtopics) {
-         print "Subtopics of $fp:\n", (join ", ", sort @subtopics), "\n";
+         print "Subtopics$fp:\n", (join ", ", sort @subtopics), "\n";
       }
-   } elsif (index($subject, "/") < 0
-              and
-            @topics = $top_help->find($subject)) {
+   } elsif (length($subject) && index($subject,"/")<0
+            and
+            @topics=$app->help->find($subject)) {
 
-      if (@topics == 1) {
+      if (@topics==1) {
          print substr($topics[0]->full_path,1), ":\n", $topics[0]->display_text;
       } else {
          print "There are ", scalar(@topics), " help topics matching '$subject':\n";
-         my $n = 0;
-         if (@topics < 5) {
+         my $n=0;
+         if (@topics<5) {
             foreach (@topics) {
                print $help_delim, ++$n, ": ", substr($_->full_path,1), ":\n",
                      $_->display_text, "\n";
@@ -1043,7 +1017,7 @@ sub help {
          } else {
             $Shell->fill_history({ temporary => 1 },
                map {
-                  $subject = substr($_->full_path, 1);
+                  $subject=substr($_->full_path,1);
                   print ++$n, ": $subject\n";
                   "help '$subject';"
                } @topics);
@@ -1058,10 +1032,10 @@ sub help {
 }
 ###############################################################################################
 sub apropos {
-   my $expr = shift;
-   $expr = qr/$expr/i;
-   if (my @list = uniq( map { $_->list_matching_leaves(sub { length($_[0]->text) && $_[0]->name =~ $expr }) }
-                        map { $_->help, @{$_->help->related } } $application, values %{$application->used} )) {
+   my $expr=shift;
+   $expr=qr/$expr/i;
+   if (my @list=uniq( map { $_->list_matching_leaves(sub { length($_[0]->text) && $_[0]->name =~ $expr }) }
+                          map { $_->help, @{$_->help->related } } $application, values %{$application->used} )) {
       print map { $_->full_path."\n" } @list;
    } else {
       print "No matching help items found\n";
@@ -1080,30 +1054,30 @@ sub show_preferences {
 }
 ###############################################################################################
 sub show_credits {
-   my $brief = shift;
-   my ($header_shown, $hidden) = (0, 0);
+   my $brief=shift;
+   my ($header_shown, $hidden)=(0,0);
    foreach my $credit (sorted_uniq(sort { ($b->product =~ /^_/) - ($a->product =~ /^_/) || $a->product cmp $b->product }
                                    map { values(%{$_->credits}), grep { defined } map { $_->credit } @{$_->extensions} } $application, values %{$application->used} )) {
 
-      my $internal = $credit->product =~ /^_/;
+      my $internal= $credit->product =~ /^_/;
 
-      if ($credit->shown == Core::Rule::Credit::hide) {
-         $hidden = true;
+      if ($credit->shown == $Core::Rule::Credit::hide) {
+         $hidden=1;
       } else {
           if (!$header_shown && !$internal) {
-             print "\nApplication ", $application->name, " currently uses following contributed and third-party software packages:\n";
+             print "\nApplication ", $application->name, " currently uses following third-party software packages:\n";
           }
           if ($brief) {
               if (!$internal) {
                  print ", " if $header_shown;
                  print $credit->product;
-                 $header_shown = true;
+                 $header_shown=1;
               }
           } else {
-              my $text = $credit->text;
+              my $text=$credit->text;
               Core::InteractiveCommands::clean_text($text);
               print "\n", substr($credit->product, $internal), "\n", $text;
-              $header_shown = !$internal;
+              $header_shown=!$internal;
           }
       }
    }
@@ -1166,14 +1140,14 @@ sub found_extension {
       $ext_dir = Cwd::abs_path($ext_dir);
       new_standalone Core::Extension($ext_dir);
       push @extensions, $ext_dir;
-      $Core::Prefs->custom->set('@extensions');
+      $Prefs->custom->set('@extensions');
    }
 }
 ###############################################################################################
 sub extend_application {
-   my $where = shift;
-   my $app = application(@_);
-   my $ext = lookup Core::Extension($where);
+   my $where=shift;
+   my $app=application(@_);
+   my $ext=lookup Core::Extension($where);
    if ($ext->is_bundled) {
       $DeveloperMode or die "A bundled extension can only be expanded from within a git working copy of polymake source tree.\n";
    }
@@ -1183,10 +1157,10 @@ sub extend_application {
 ###############################################################################################
 sub found_application {
    my ($where, $app_name)=@_;
-   my $app = lookup Core::Application($app_name);
+   my $app=lookup Core::Application($app_name);
    my $exists_in;
    if ($app) {
-      $exists_in = $app->installTop;
+      $exists_in=$app->installTop;
    } else {
       foreach ($InstallTop, @extensions) {
          if (-d "$_/apps/$app_name") {
@@ -1208,7 +1182,7 @@ sub found_application {
       if ($exists_in) {
          die "Application $app_name already exists ", $exists_in eq $InstallTop ? "in the polymake core" : "in extension $exists_in", "\n";
       }
-      my $ext = lookup Core::Extension($where);
+      my $ext=lookup Core::Extension($where);
       if ($ext->is_bundled) {
          die "A new application can't be founded in a bundled extension; please specify \"core\" as location instead.";
       } else {
@@ -1219,11 +1193,11 @@ sub found_application {
 }
 ###############################################################################################
 sub import_extension {
-   my $ext_dir = shift;
+   my $ext_dir=shift;
    replace_special_paths($ext_dir);
    -d $ext_dir or die "Directory $ext_dir does not exist.\n";
-   $ext_dir = Cwd::abs_path($ext_dir);
-   if (defined (my $ext = $Core::Extension::registered_by_dir{$ext_dir})) {
+   $ext_dir=Cwd::abs_path($ext_dir);
+   if (defined (my $ext=$Core::Extension::registered_by_dir{$ext_dir})) {
       if ($ext->is_active) {
          die "Extension $ext_dir is already loaded.\n";
       } else {
@@ -1245,29 +1219,29 @@ To get additional help about configuration options:
 }
 ###############################################################################################
 sub reconfigure_extension {
-   my $what = shift;
+   my $what=shift;
    lookup Core::Extension($what)->reconfigure(@_);
 }
 ###############################################################################################
 sub obliterate_extension {
-   my $what = shift;
-   my $ext = lookup Core::Extension($what);
+   my $what=shift;
+   my $ext=lookup Core::Extension($what);
    $ext->may_obliterate;
-   $Core::Prefs->custom->delete_from_private_list('@extensions', string_list_index(\@extensions, $ext->dir))
+   $Prefs->custom->delete_from_private_list('@extensions', string_list_index(\@extensions, $ext->dir))
      or die "Extension ", $ext->URI, " can't be obliterated: it is included by global configuration\n";
 
    foreach my $app ($ext->obliterate) {
       if ($app==$default_application) {
-         $Core::Prefs->custom->reset('$default_application');
+         $Prefs->custom->reset('$default_application');
       }
       if ($app==$application) {
          application($default_application);
       }
       delete_string_from_list(\@start_applications, $app->name);
    }
-   $Core::Prefs->custom->set('@start_applications');
+   $Prefs->custom->set('@start_applications');
    if (delete $disabled_extensions{$ext->dir}) {
-      $Core::Prefs->custom->set('%disabled_extensions');
+      $Prefs->custom->set('%disabled_extensions');
    }
 }
 ###############################################################################################
@@ -1283,25 +1257,24 @@ sub unconfigure {
 ###############################################################################################
 sub show_unconfigured {
    my (%shown_credits, $credit, $shown);
-   foreach my $app ($application, values %{$application->used}) {
+   foreach my $rule_key ($application->list_configured(-1)) {
+      print "$rule_key\n",
+            ($credit=$application->first_credit_in($rule_key) and !($shown_credits{$credit->product}++))
+            ? ($credit->text, "\n") : ();
+      $shown=1;
+   }
+   while (my ($name, $app)=each %{$application->used}) {
       foreach my $rule_key ($app->list_configured(-1)) {
-         print $app != $application ? $app->name."::" : "", $rule_key, "\n",
-               ($credit = $app->first_credit_in($rule_key) and !($shown_credits{$credit->product}++))
+         print "$name\::$rule_key\n",
+               ($credit=$app->first_credit_in($rule_key) and !($shown_credits{$credit->product}++))
                ? ($credit->text, "\n") : ();
-         $shown = true;
+         $shown=1;
       }
    }
    if ($shown) {
       print "\nTo enable an interface:  reconfigure(\"", Core::InteractiveCommands::underline("RULEFILE"), "\");\n";
    }
 }
-###############################################################################################
-sub export_configured {
-   my $filename = shift;
-   my $opts = @_==1 && ref($_[0]) eq "HASH" ? shift : { @_ };
-   replace_special_paths($filename);
-   $Core::Custom->export($filename, $opts, $Core::Prefs->custom);
-}
 
 1
 
