@@ -1,7 +1,6 @@
-#  Copyright (c) 1997-2019
-#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
-#  Technische Universität Berlin, Germany
-#  https://polymake.org
+#  Copyright (c) 1997-2018
+#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
+#  http://www.polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -65,10 +64,10 @@ sub delete_from_list {  # \@list, numeric scalar or ref => true if deleted
    for (my $i=$#$list; $i>=0; --$i) {
       if ($list->[$i]==$item) {
          splice @$list, $i, 1;
-         return true;
+         return 1;
       }
    }
-   false
+   0
 }
 ####################################################################################
 sub delete_string_from_list {   # \@list, string scalar => true if deleted
@@ -76,26 +75,26 @@ sub delete_string_from_list {   # \@list, string scalar => true if deleted
    for (my $i=$#$list; $i>=0; --$i) {
       if ($list->[$i] eq $item) {
          splice @$list, $i, 1;
-         return true;
+         return 1;
       }
    }
-   false
+   0
 }
 ####################################################################################
 sub contains {          # \@list, numeric scalar or ref => bool
    my ($list, $item)=@_;
    foreach my $elem (@$list) {
-      $elem==$item and return true;
+      $elem==$item and return 1;
    }
-   return false;
+   return 0;
 }
 ####################################################################################
 sub contains_string {           # \@list, scalar => bool
    my ($list, $item)=@_;
    foreach my $elem (@$list) {
-      $elem eq $item and return true;
+      $elem eq $item and return 1;
    }
-   return false;
+   return 0;
 }
 ####################################################################################
 # "string1", "string2" [, "delimiter" ]
@@ -135,9 +134,9 @@ sub equal_lists {
    my $end=@$l1;
    return 0 if $end!=@$l2;
    for (my $i=0; $i<$end; ++$i) {
-       $l1->[$i] == $l2->[$i] or return false;
+       $l1->[$i] == $l2->[$i] or return 0;
    }
-   true
+   1
 }
 
 sub equal_string_lists {
@@ -145,9 +144,9 @@ sub equal_string_lists {
    my $end=@$l1;
    return 0 if $end!=@$l2;
    for (my $i=0; $i<$end; ++$i) {
-       $l1->[$i] eq $l2->[$i] or return false;
+       $l1->[$i] eq $l2->[$i] or return 0;
    }
-   true
+   1
 }
 
 # \list1, start_index1, \list2, start_index2 => length of the equal sequence starting at given positions
@@ -189,32 +188,7 @@ sub equal_list_prefixes2 {
    }
    $ret
 }
-####################################################################################
-sub equal_hashes {
-   my ($h1, $h2)=@_;
-   if (keys(%$h1) == keys(%$h2)) {
-      while (my ($k, $v)=each %$h1) {
-         unless (exists $h2->{$k} and defined($v) ? $h2->{$k}==$v : !defined($h2->{$k})) {
-            keys %$h1;
-            return false;
-         }
-      }
-      true;
-   }
-}
 
-sub equal_string_hashes {
-   my ($h1, $h2)=@_;
-   if (keys(%$h1) == keys(%$h2)) {
-      while (my ($k, $v)=each %$h1) {
-         unless (exists $h2->{$k} and defined($v) ? $h2->{$k} eq $v : !defined($h2->{$k})) {
-            keys %$h1;
-            return false;
-         }
-      }
-      true;
-   }
-}
 ####################################################################################
 sub enforce_nl($) {
    $_[0].="\n" if substr($_[0],-1) ne "\n";
@@ -245,7 +219,7 @@ sub warn_print {
 sub croak {
    my ($pkg, $file, $line, $sub);
    my $i=0;
-   if (is_object($INC[0]) && ref($INC[0]) eq "Polymake::Core::Application" && defined($INC[0]->compile_scope)) {
+   if (is_object($INC[0]) && defined($INC[0]->compile_scope)) {
       do {
          ($pkg, $file, $line, $sub)=caller(++$i);
       } while ($pkg =~ /^Polymake(?:$|::Core::|::Overload\b|::Struct\b)/ || $sub =~ /::check_object_pkg$/ );
@@ -283,6 +257,8 @@ sub croak {
 
 sub beautify_error {
    unless ($DebugLevel) {
+      # FIXME: remove this!
+      namespaces::temp_disable();
       use re 'eval';
       $@ =~ s/ at \(eval \d+\)(?:\[.*?:\d+\])? line 1(\.)?/$1/g;
       $@ =~ s/, <\$?$id_re> line \d+\.//go;
